@@ -101,13 +101,13 @@ display_usage (progname, omit_vals, appendage, fonts)
     fprintf (stdout, "\n\
 To list available fonts, type `%s -T \"format\" --help-fonts',\n\
 where \"format\" is the output format:\n\
-X, pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
+X, pnm, gif, ai, ps, cgm, fig, pcl, hpgl, or tek.\n",
 	     progname);
 #else  /* X_DISPLAY_MISSING */
     fprintf (stdout, "\n\
 To list available fonts, type `%s -T \"format\" --help-fonts',\n\
 where \"format\" is the output format:\n\
-gif, pnm, ai, ps, fig, pcl, hpgl, or tek.\n",
+gif, pnm, ai, ps, cgm, fig, pcl, hpgl, or tek.\n",
 	     progname);
 #endif /* X_DISPLAY_MISSING */
 
@@ -128,7 +128,7 @@ display_version (progname)
   fprintf (stdout, 
 	   "%s (GNU %s) %s\n", progname, PACKAGE, VERSION);
   fprintf (stdout, 
-	   "Copyright (C) 1989-1998 Free Software Foundation, Inc.\n");
+	   "Copyright (C) 1989-1999 Free Software Foundation, Inc.\n");
   fprintf (stdout, 
 	   "The GNU %s package comes with NO WARRANTY, to the extent permitted\n", PACKAGE);
   fprintf (stdout, 
