@@ -1,30 +1,35 @@
-/* This file contains the display_fonts routine, which is used in
-   user-level executables that are linked with libplot.  Its output is
-   device-specific. */
+/* This file contains the display_fonts and list_fonts routines, which are
+   used in user-level executables that are linked with libplot.  Their
+   output is device-specific.
+
+   Currently, they get information about font names by invoking
+   undocumented members of the libplot API, which return pointers to
+   internal library data structures. */
 
 #include "sys-defines.h"
 #include "plot.h"
-#include "plotcompat.h"
 
 /* forward references */
 int display_fonts ____P((const char *display_type, const char *progname));
 int list_fonts ____P((const char *display_type, const char *progname));
 
 /* for use in printing font names in two columns; assumption is that all
-   font names have lengths in range 0..MAX_FONTNAME_LEN, inclusive */
+   font name strings have lengths in range 0..MAX_FONTNAME_LEN inclusive
+   (not counting final null) */
 #define MAX_FONTNAME_LEN 36
 static char spaces[MAX_FONTNAME_LEN+1] = "                                   ";
 
 /* The definitions of these structures are taken from ../libplot/extern.h.
    IF THOSE STRUCTURES CHANGE, THESE SHOULD TOO.
 
-   Font information is stored in ../libplot/g_fontdb.c. */
+   Font information is stored in ../libplot/g_fontdb.c, and we'll retrieve
+   pointers to it. */
 
-struct hershey_font_info_struct 
+struct plHersheyFontInfoStruct 
 {
-  char *name;			/* font name */
-  char *othername;		/* an alias (for backward compatibility) */
-  char *orig_name;		/* Allen Hershey's original name for it */
+  const char *name;		/* font name */
+  const char *othername;	/* an alias (for backward compatibility) */
+  const char *orig_name;	/* Allen Hershey's original name for it */
   short chars[256];		/* array of vector glyphs */
   int typeface_index;		/* default typeface for the font */
   int font_index;		/* which font within typeface this is */
@@ -32,12 +37,12 @@ struct hershey_font_info_struct
   bool iso8859_1;		/* whether font encoding is iso8859-1 */
   bool visible;		/* whether font is visible, i.e. not internal */
 };
-extern struct hershey_font_info_struct _hershey_font_info[];
 
-struct ps_font_info_struct 
+struct plPSFontInfoStruct
 {
   const char *ps_name;		/* the postscript font name */
   const char *ps_name_alt;	/* alternative PS font name, if non-NULL */
+  const char *ps_name_alt2;	/* 2nd alternative PS font name, if non-NULL */
   const char *x_name;		/* the X Windows font name */
   const char *x_name_alt;	/* alternative X Windows font name */
   int pcl_typeface;		/* the PCL typeface number */
@@ -47,6 +52,7 @@ struct ps_font_info_struct
   int pcl_symbol_set;		/* 0=Roman-8, 14=ISO-8859-1, etc. */
   int font_ascent;		/* the font's ascent (from bounding box) */
   int font_descent;		/* the font's descent (from bounding box) */
+  int font_cap_height;		/* the font's cap height */
   short width[256];		/* per-character width information */
   short offset[256];		/* per-character left edge information */
   int typeface_index;		/* default typeface for the font */
@@ -55,12 +61,11 @@ struct ps_font_info_struct
   bool iso8859_1;		/* whether font encoding is iso8859-1 */
 };
 
-extern struct ps_font_info_struct _ps_font_info[];
-
-struct pcl_font_info_struct 
+struct plPCLFontInfoStruct
 {
   const char *ps_name;		/* the postscript font name */
-  const char *substitute_ps_name; /* alt. name when in a PS file, if non-NULL */
+  const char *ps_name_alt;	/* alternative PS font name, if non-NULL */
+  const char *substitute_ps_name; /* replacement name (for use in a PS file) */
   const char *x_name;		/* the X Windows font name */
   int pcl_typeface;		/* the PCL typeface number */
   int pcl_spacing;		/* 0=fixed width, 1=variable */
@@ -76,9 +81,7 @@ struct pcl_font_info_struct
   bool iso8859_1;		/* whether font encoding is iso8859-1 */
 };
 
-extern struct pcl_font_info_struct _pcl_font_info[];
-
-struct stick_font_info_struct 
+struct plStickFontInfoStruct 
 {
   const char *ps_name;		/* the postscript font name */
 				/* no x_name field */  
@@ -106,16 +109,14 @@ struct stick_font_info_struct
   bool iso8859_1;		/* encoding is iso8859-1? (after reencoding) */
 };
 
-extern const struct stick_font_info_struct _stick_font_info[];
-
 /* List of Plotter types we support getting font information from,
    NULL-terminated.  This list also appears in the program text below. */
 #ifndef X_DISPLAY_MISSING
-static char *_known_devices[] =
-{ "X", "pnm", "gif", "ai", "ps", "fig", "pcl", "hpgl", "tek", "meta", NULL };
+static const char *_known_devices[] =
+{ "X", "pnm", "gif", "ai", "ps", "cgm", "fig", "pcl", "hpgl", "tek", "meta", NULL };
 #else
-static char *_known_devices[] =
-{ "pnm", "gif", "ai", "ps", "fig", "pcl", "hpgl", "tek", "meta", NULL };
+static const char *_known_devices[] =
+{ "pnm", "gif", "ai", "ps", "cgm", "fig", "pcl", "hpgl", "tek", "meta", NULL };
 #endif
 
 int
@@ -126,9 +127,11 @@ display_fonts (display_type, progname)
      const char *display_type, *progname;
 #endif
 {
-  int numfonts, numpairs, i, j, k, handle;
+  plPlotter *plotter;
+  plPlotterParams *plotter_params;
+  int numfonts, numpairs, i, j, k;
   bool found = false, odd;
-  char **device_ptr = _known_devices;
+  const char **device_ptr = _known_devices;
 
   while (*device_ptr)
     if (strcmp (display_type, *device_ptr++) == 0)
@@ -143,47 +146,50 @@ display_fonts (display_type, progname)
       fprintf (stderr, "\
 To list available fonts, type `%s -T \"format\" --help-fonts',\n\
 where \"format\" is the output format:\n\
-X, pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
+X, pnm, gif, ai, ps, cgm, fig, pcl, hpgl, or tek.\n",
 	       progname);
 #else  /* X_DISPLAY_MISSING */
       fprintf (stderr, "\
 To list available fonts, type `%s -T \"format\" --help-fonts',\n\
 where \"format\" is the output format:\n\
-pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
+pnm, gif, ai, ps, cgm, fig, pcl, hpgl, or tek.\n",
 	       progname);
 #endif /* X_DISPLAY_MISSING */
       return 0;
     }
 
-  if ((handle = newpl (display_type, NULL, stdout, stderr)) < 0)
+  plotter_params = pl_newplparams ();
+  if ((plotter = pl_newpl_r (display_type, NULL, stdout, stderr,
+			     plotter_params)) == NULL)
     {
       fprintf (stderr, 
 	       "%s: no font information on display device \"%s\" is available\n",
 	       progname, display_type);
       return 0;
     }
-  else
-    selectpl (handle);
 
-  if (havecap ("HERSHEY_FONTS"))
+  if (pl_havecap_r (plotter, "HERSHEY_FONTS"))
     {
+      const struct plHersheyFontInfoStruct *hershey_font_info = 
+	(const struct plHersheyFontInfoStruct *)pl_get_hershey_font_info (plotter);
       int visible_num;
 
       numfonts = 0;
-      for (i=0; _hershey_font_info[i].name; i++)
-	if (_hershey_font_info[i].visible)
+      for (i=0; hershey_font_info[i].name; i++)
+	if (hershey_font_info[i].visible)
 	numfonts++;
       odd = (numfonts % 2 == 1 ? true : false);
       numpairs = numfonts / 2;
 
-      /* compute j and k: j=0, k=numpairs + (odd ? 1 : 0) in terms of visibles */
+      /* compute j and k: j=0, k=numpairs + (odd ? 1 : 0) in terms of
+         visibles */
       j = 0;
       k = 0;
       visible_num = -1;
-      for (i=0; _hershey_font_info[i].name; i++)
-	if (_hershey_font_info[i].visible)
+      for (i=0; hershey_font_info[i].name; i++)
+	if (hershey_font_info[i].visible)
 	  {
-	    visible_num++;	/* visible_num is index into array of visibles */
+	    visible_num++;  /* visible_num is index into array of visibles */
 	    if (visible_num == 0)
 	      j = i;
 	    else if (visible_num == numpairs + (odd ? 1 : 0))
@@ -196,41 +202,43 @@ pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
 	{
 	  int len;
 	  
-	  len = strlen (_hershey_font_info[j].name);
-	  fprintf (stdout, "\t%s", _hershey_font_info[j].name);
+	  len = strlen (hershey_font_info[j].name);
+	  fprintf (stdout, "\t%s", hershey_font_info[j].name);
 	  spaces[MAX_FONTNAME_LEN - len] = '\0';
 	  fputs (spaces, stdout);
 	  spaces[MAX_FONTNAME_LEN - len] = ' ';
-	  fprintf (stdout, "%s\n", _hershey_font_info[k].name);
+	  fprintf (stdout, "%s\n", hershey_font_info[k].name);
 	  /* bump both j and k */
 	  do
 	    j++;
-	  while (_hershey_font_info[j].visible == false);
+	  while (hershey_font_info[j].visible == false);
 	  if (i < numpairs - 1)
 	    {
 	      do
 		k++;
-	      while (_hershey_font_info[k].visible == false);
+	      while (hershey_font_info[k].visible == false);
 	    }
 	}
       if (odd)
-	fprintf (stdout, "\t%s\n", _hershey_font_info[j].name);
+	fprintf (stdout, "\t%s\n", hershey_font_info[j].name);
     }
 
-  if (havecap ("STICK_FONTS"))
+  if (pl_havecap_r (plotter, "STICK_FONTS"))
     {
+      const struct plStickFontInfoStruct *stick_font_info = 
+	(const struct plStickFontInfoStruct *)pl_get_stick_font_info (plotter);
       int extra_fonts, *goodfonts;
 
       numfonts = 0;
-      for (i=0; _stick_font_info[i].ps_name; i++)
+      for (i=0; stick_font_info[i].ps_name; i++)
 	numfonts++;
 
       /* if this Plotter doesn't support extras, skip them */
-      extra_fonts = havecap ("EXTRA_STICK_FONTS");
+      extra_fonts = pl_havecap_r (plotter, "EXTRA_STICK_FONTS");
       goodfonts = (int *)malloc (numfonts * sizeof(int));
-      for (i=0, j=0; _stick_font_info[i].ps_name; i++)
+      for (i=0, j=0; stick_font_info[i].ps_name; i++)
 	{
-	  if (!extra_fonts && _stick_font_info[i].basic == false)
+	  if (!extra_fonts && stick_font_info[i].basic == false)
 	    continue;
 	  goodfonts[j++] = i;
 	}
@@ -245,23 +253,26 @@ pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
 	{
 	  int len;
 	  
-	  len = strlen (_stick_font_info[goodfonts[j]].ps_name);
-	  fprintf (stdout, "\t%s", _stick_font_info[goodfonts[j++]].ps_name);
+	  len = strlen (stick_font_info[goodfonts[j]].ps_name);
+	  fprintf (stdout, "\t%s", stick_font_info[goodfonts[j++]].ps_name);
 	  spaces[MAX_FONTNAME_LEN - len] = '\0';
 	  fputs (spaces, stdout);
 	  spaces[MAX_FONTNAME_LEN - len] = ' ';
-	  fprintf (stdout, "%s\n", _stick_font_info[goodfonts[k++]].ps_name);
+	  fprintf (stdout, "%s\n", stick_font_info[goodfonts[k++]].ps_name);
 	}
       if (odd)
-	fprintf (stdout, "\t%s\n", _stick_font_info[goodfonts[j]].ps_name);
+	fprintf (stdout, "\t%s\n", stick_font_info[goodfonts[j]].ps_name);
 
       free (goodfonts);
     }
 
-  if (havecap ("PCL_FONTS"))
+  if (pl_havecap_r (plotter, "PCL_FONTS"))
     {
+      const struct plPCLFontInfoStruct *pcl_font_info = 
+	(const struct plPCLFontInfoStruct *)pl_get_pcl_font_info (plotter);
+
       numfonts = 0;
-      for (i=0; _pcl_font_info[i].ps_name; i++)
+      for (i=0; pcl_font_info[i].ps_name; i++)
 	numfonts++;
       odd = (numfonts % 2 == 1 ? true : false);
       numpairs = numfonts / 2;
@@ -272,21 +283,23 @@ pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
 	{
 	  int len;
 	  
-	  len = strlen (_pcl_font_info[j].ps_name);
-	  fprintf (stdout, "\t%s", _pcl_font_info[j++].ps_name);
+	  len = strlen (pcl_font_info[j].ps_name);
+	  fprintf (stdout, "\t%s", pcl_font_info[j++].ps_name);
 	  spaces[MAX_FONTNAME_LEN - len] = '\0';
 	  fputs (spaces, stdout);
 	  spaces[MAX_FONTNAME_LEN - len] = ' ';
-	  fprintf (stdout, "%s\n", _pcl_font_info[k++].ps_name);
+	  fprintf (stdout, "%s\n", pcl_font_info[k++].ps_name);
 	}
       if (odd)
-	fprintf (stdout, "\t%s\n", _pcl_font_info[j].ps_name);
+	fprintf (stdout, "\t%s\n", pcl_font_info[j].ps_name);
     }
 
-  if (havecap ("PS_FONTS"))
+  if (pl_havecap_r (plotter, "PS_FONTS"))
     {
+      const struct plPSFontInfoStruct *ps_font_info = 
+	(const struct plPSFontInfoStruct *)pl_get_ps_font_info (plotter);
       numfonts = 0;
-      for (i=0; _ps_font_info[i].ps_name; i++)
+      for (i=0; ps_font_info[i].ps_name; i++)
 	numfonts++;
       odd = (numfonts % 2 == 1 ? true : false);
       numpairs = numfonts / 2;
@@ -297,15 +310,15 @@ pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
 	{
 	  int len;
 	  
-	  len = strlen (_ps_font_info[j].ps_name);
-	  fprintf (stdout, "\t%s", _ps_font_info[j++].ps_name);
+	  len = strlen (ps_font_info[j].ps_name);
+	  fprintf (stdout, "\t%s", ps_font_info[j++].ps_name);
 	  spaces[MAX_FONTNAME_LEN - len] = '\0';
 	  fputs (spaces, stdout);
 	  spaces[MAX_FONTNAME_LEN - len] = ' ';
-	  fprintf (stdout, "%s\n", _ps_font_info[k++].ps_name);
+	  fprintf (stdout, "%s\n", ps_font_info[k++].ps_name);
 	}
       if (odd)
-	fprintf (stdout, "\t%s\n", _ps_font_info[j].ps_name);
+	fprintf (stdout, "\t%s\n", ps_font_info[j].ps_name);
     }
 
   if (strcmp (display_type, "X") == 0)
@@ -317,6 +330,9 @@ pnm, gif, ai, ps, fig, pcl, hpgl, or tek.\n",
   return 1;
 }
 
+/* Write font names to standard output, in a line-by-line rather than a
+   tabular form. */
+
 int
 #ifdef _HAVE_PROTOS
 list_fonts (const char *display_type, const char *progname)
@@ -325,9 +341,11 @@ list_fonts (display_type, progname)
      const char *display_type, *progname;
 #endif
 {
+  plPlotter *plotter;
+  plPlotterParams *plotter_params;
   bool found = false;
-  int i, handle;
-  char **device_ptr = _known_devices;
+  int i;
+  const char **device_ptr = _known_devices;
 
   while (*device_ptr)
     if (strcmp (display_type, *device_ptr++) == 0)
@@ -336,33 +354,64 @@ list_fonts (display_type, progname)
 	break;
       }
 
-  if (found == false || 
-      (handle = newpl (display_type, NULL, stdout, stderr)) < 0)
+  if (found == false)
     {
       fprintf (stderr, 
 	       "%s: no font information on display device \"%s\" is available\n",
 	       progname, display_type);
       return 0;
     }
-  else
-    selectpl (handle);
 
-  if (havecap ("HERSHEY_FONTS"))
-    for (i=0; _hershey_font_info[i].name; i++)
-      if (_hershey_font_info[i].visible)
-	fprintf (stdout, "%s\n", _hershey_font_info[i].name);
+  plotter_params = pl_newplparams ();
+  if ((plotter = pl_newpl_r (display_type, NULL, stdout, stderr,
+			     plotter_params)) == NULL)
+    {
+      fprintf (stderr, 
+	       "%s: no font information on display device \"%s\" is available\n",
+	       progname, display_type);
+      return 0;
+    }
 
-  if (havecap ("PCL_FONTS"))
-    for (i=0; _pcl_font_info[i].ps_name; i++)
-      fprintf (stdout, "%s\n", _pcl_font_info[i].ps_name);
+  if (pl_havecap_r (plotter, "HERSHEY_FONTS"))
+    {
+      const struct plHersheyFontInfoStruct *hershey_font_info = 
+	(const struct plHersheyFontInfoStruct *)pl_get_hershey_font_info (plotter);
+      for (i=0; hershey_font_info[i].name; i++)
+	if (hershey_font_info[i].visible)
+	  fprintf (stdout, "%s\n", hershey_font_info[i].name);
+    }
 
-  if (havecap ("STICK_FONTS"))
-    for (i=0; _stick_font_info[i].ps_name; i++)
-      fprintf (stdout, "%s\n", _stick_font_info[i].ps_name);
+  if (pl_havecap_r (plotter, "STICK_FONTS"))
+    {
+      const struct plStickFontInfoStruct *stick_font_info = 
+	(const struct plStickFontInfoStruct *)pl_get_stick_font_info (plotter);
+      int extra_fonts = pl_havecap_r (plotter, "EXTRA_STICK_FONTS");
 
-  if (havecap ("PS_FONTS"))
-    for (i=0; _ps_font_info[i].ps_name; i++)
-      fprintf (stdout, "%s\n", _ps_font_info[i].ps_name);
+      for (i=0; stick_font_info[i].ps_name; i++)
+	{
+	  if (!extra_fonts && stick_font_info[i].basic == false)
+	    continue;
+	  fprintf (stdout, "%s\n", stick_font_info[i].ps_name);
+	}
+    }
+
+  if (pl_havecap_r (plotter, "PCL_FONTS"))
+    {
+      const struct plPCLFontInfoStruct *pcl_font_info = 
+	(const struct plPCLFontInfoStruct *)pl_get_pcl_font_info (plotter);
+
+      for (i=0; pcl_font_info[i].ps_name; i++)
+	fprintf (stdout, "%s\n", pcl_font_info[i].ps_name);
+    }
+      
+  if (pl_havecap_r (plotter, "PS_FONTS"))
+    {
+      const struct plPSFontInfoStruct *ps_font_info = 
+	(const struct plPSFontInfoStruct *)pl_get_ps_font_info (plotter);
+
+      for (i=0; ps_font_info[i].ps_name; i++)
+	fprintf (stdout, "%s\n", ps_font_info[i].ps_name);
+    }
 
   return 1;
 }
