@@ -7,13 +7,14 @@
 
 void
 #ifdef _HAVE_PROTOS
-_a_set_pen_color(void)
+_a_set_pen_color(S___(Plotter *_plotter))
 #else
-_a_set_pen_color()
+_a_set_pen_color(S___(_plotter))
+     S___(Plotter *_plotter;)
 #endif
 {
   double red, green, blue;
-  double cyan, magenta, yellow, black;
+  double cyan, magenta, yellow, black, temp;
 
   /* convert from RGB to CMYK */
   red = ((double)((_plotter->drawstate->fgcolor).red))/0xFFFF;
@@ -22,7 +23,8 @@ _a_set_pen_color()
   cyan = 1.0 - red;
   magenta = 1.0 - green;
   yellow = 1.0 - blue;
-  black = DMIN(cyan,DMIN(magenta,yellow));
+  temp = magenta < yellow ? magenta : yellow;
+  black = cyan < temp ? cyan : temp;
   cyan -= black;
   magenta -= black;
   yellow -= black;
@@ -55,34 +57,35 @@ _a_set_pen_color()
   return;
 }
 
-/* fill_level, if nonzero, specifies the extent to which the nominal fill
+/* fill_type, if nonzero, specifies the extent to which the nominal fill
    color should be desaturated.  1 means no desaturation, 0xffff means
    complete desaturation (white).
 
-   This may be invoked if fill_level is 0.  If so, it's a sign that the
+   This may be invoked if fill_type is 0.  If so, it's a sign that the
    nominal fill color has temporarily been set to the current pen color
    (for a subtle reason; see comment in a_endpath.c).  So in that case, we
    treat 0 as 1. */
 void
 #ifdef _HAVE_PROTOS
-_a_set_fill_color(void)
+_a_set_fill_color(S___(Plotter *_plotter))
 #else
-_a_set_fill_color()
+_a_set_fill_color(S___(_plotter))
+     S___(Plotter *_plotter;)
 #endif
 {
-  int fill_level;
+  int fill_type;
   double red, green, blue;
-  double cyan, magenta, yellow, black;
+  double cyan, magenta, yellow, black, temp;
   double desaturate;
 
   red = ((double)((_plotter->drawstate->fillcolor).red))/0xFFFF;
   green = ((double)((_plotter->drawstate->fillcolor).green))/0xFFFF;
   blue = ((double)((_plotter->drawstate->fillcolor).blue))/0xFFFF;
 
-  fill_level = _plotter->drawstate->fill_level;
-  if (fill_level == 0)
-    fill_level = 1;		/* see comment above */
-  desaturate = ((double)fill_level - 1.)/0xFFFE;
+  fill_type = _plotter->drawstate->fill_type;
+  if (fill_type == 0)
+    fill_type = 1;		/* see comment above */
+  desaturate = ((double)fill_type - 1.)/0xFFFE;
   red = red + desaturate * (1.0 - red);
   green = green + desaturate * (1.0 - green);
   blue = blue + desaturate * (1.0 - blue);
@@ -91,7 +94,8 @@ _a_set_fill_color()
   cyan = 1.0 - red;
   magenta = 1.0 - green;
   yellow = 1.0 - blue;
-  black = DMIN(cyan,DMIN(magenta,yellow));
+  temp = magenta < yellow ? magenta : yellow;
+  black = cyan < temp ? cyan : temp;
   cyan -= black;
   magenta -= black;
   yellow -= black;
