@@ -12,9 +12,10 @@
 
 int 
 #ifdef _HAVE_PROTOS
-_f_flinewidth (double new_line_width)
+_f_flinewidth (R___(Plotter *_plotter) double new_line_width)
 #else
-_f_flinewidth (new_line_width)
+_f_flinewidth (R___(_plotter) new_line_width)
+     S___(Plotter *_plotter;)
      double new_line_width;
 #endif
 {
@@ -23,12 +24,12 @@ _f_flinewidth (new_line_width)
 
   if (!_plotter->open)
     {
-      _plotter->error ("flinewidth: invalid operation");
+      _plotter->error (R___(_plotter) "flinewidth: invalid operation");
       return -1;
     }
 
   /* invoke generic method */
-  _g_flinewidth(new_line_width);
+  _g_flinewidth (R___(_plotter) new_line_width);
 
   /* xfig expresses line widths in terms of `Fig display units', so we
      scale appropriately */
