@@ -12,7 +12,7 @@
 
 /* can declare this array `extern', so that applications linked with
    libplot/libplotter can secretly access it, if they want to :-) */
-const Colornameinfo _colornames[NUM_COLORNAMES + 1] =
+const plColorNameInfo _colornames[NUM_COLORNAMES + 1] =
 {
   {"aliceblue",		0xf0, 0xf8, 0xff},
   {"antiquewhite",	0xfa, 0xeb, 0xd7},
