@@ -8,15 +8,17 @@
 
 int
 #ifdef _HAVE_PROTOS
-_g_fline (double x0, double y0, double x1, double y1)
+_g_fline (R___(Plotter *_plotter) double x0, double y0, double x1, double y1)
 #else
-_g_fline (x0, y0, x1, y1)
+_g_fline (R___(_plotter) x0, y0, x1, y1)
+     S___(Plotter *_plotter;) 
      double x0, y0, x1, y1;
 #endif
 {
   if (!_plotter->open)
     {
-      _plotter->error ("fline: invalid operation");
+      _plotter->error (R___(_plotter) 
+		       "fline: invalid operation");
       return -1;
     }
 
@@ -25,7 +27,7 @@ _g_fline (x0, y0, x1, y1)
      move() method invokes the endpath() method). */
   if (x0 != _plotter->drawstate->pos.x
       || y0 != _plotter->drawstate->pos.y)
-    _plotter->fmove (x0, y0);
+    _plotter->fmove (R___(_plotter) x0, y0);
 
-  return _plotter->fcont (x1, y1);
+  return _plotter->fcont (R___(_plotter) x1, y1);
 }
