@@ -6,19 +6,21 @@
 
 int
 #ifdef _HAVE_PROTOS
-_m_erase (void)
+_m_erase (S___(Plotter *_plotter))
 #else
-_m_erase ()
+_m_erase (S___(_plotter))
+     S___(Plotter *_plotter;) 
 #endif
 {
   if (!_plotter->open)
     {
-      _plotter->error ("erase: invalid operation");
+      _plotter->error (R___(_plotter) 
+		       "erase: invalid operation");
       return -1;
     }
 
-  _meta_emit_byte ((int)O_ERASE);
-  _meta_emit_terminator ();
+  _meta_emit_byte (R___(_plotter) (int)O_ERASE);
+  _meta_emit_terminator (S___(_plotter));
 
   /* on to next frame */
   _plotter->frame_number++;
