@@ -13,24 +13,26 @@
 
 int
 #ifdef _HAVE_PROTOS
-_m_fillcolor (int red, int green, int blue)
+_m_fillcolor (R___(Plotter *_plotter) int red, int green, int blue)
 #else
-_m_fillcolor (red, green, blue)
+_m_fillcolor (R___(_plotter) red, green, blue)
+     S___(Plotter *_plotter;) 
      int red, green, blue;
 #endif
 {
   if (!_plotter->open)
     {
-      _plotter->error ("fillcolor: invalid operation");
+      _plotter->error (R___(_plotter) 
+		       "fillcolor: invalid operation");
       return -1;
     }
 
-  _meta_emit_byte ((int)O_FILLCOLOR);
-  _meta_emit_integer (red);
-  _meta_emit_integer (green);
-  _meta_emit_integer (blue);
-  _meta_emit_terminator ();
+  _meta_emit_byte (R___(_plotter) (int)O_FILLCOLOR);
+  _meta_emit_integer (R___(_plotter) red);
+  _meta_emit_integer (R___(_plotter) green);
+  _meta_emit_integer (R___(_plotter) blue);
+  _meta_emit_terminator (S___(_plotter));
 
   /* invoke generic method */
-  return _g_fillcolor (red, green, blue);
+  return _g_fillcolor (R___(_plotter) red, green, blue);
 }
