@@ -17,22 +17,24 @@
 
 int
 #ifdef _HAVE_PROTOS
-_m_filltype (int level)
+_m_filltype (R___(Plotter *_plotter) int level)
 #else
-_m_filltype (level)
+_m_filltype (R___(_plotter) level)
+     S___(Plotter *_plotter;) 
      int level;
 #endif
 {
   if (!_plotter->open)
     {
-      _plotter->error ("filltype: invalid operation");
+      _plotter->error (R___(_plotter) 
+		       "filltype: invalid operation");
       return -1;
     }
 
-  _meta_emit_byte ((int)O_FILLTYPE);
-  _meta_emit_integer (level);
-  _meta_emit_terminator ();
+  _meta_emit_byte (R___(_plotter) (int)O_FILLTYPE);
+  _meta_emit_integer (R___(_plotter) level);
+  _meta_emit_terminator (S___(_plotter));
   
   /* invoke generic method */
-  return _g_filltype (level);
+  return _g_filltype (R___(_plotter) level);
 }
