@@ -6,25 +6,26 @@
 
 int
 #ifdef _HAVE_PROTOS
-_n_erase (void)
+_n_erase (S___(Plotter *_plotter))
 #else
-_n_erase ()
+_n_erase (S___(_plotter))
+     S___(Plotter *_plotter;)
 #endif
 {
   if (!_plotter->open)
     {
-      _plotter->error ("erase: invalid operation");
+      _plotter->error (R___(_plotter) "erase: invalid operation");
       return -1;
     }
 
   if (_plotter->drawstate->points_in_path > 0)
-    _plotter->endpath(); /* flush polyline if any */
+    _plotter->endpath (S___(_plotter)); /* flush polyline if any */
 
-  /* deallocate bitmap */
-  _n_delete_image ();
+  /* delete image, i.e., deallocate libxmi canvas */
+  _n_delete_image (S___(_plotter));
 
-  /* create new image, i.e. bitmap, initialized to background color */
-  _n_new_image ();
+  /* create new image, i.e. libxmi canvas, initialized to background color */
+  _n_new_image (S___(_plotter));
   
   /* on to next frame (for a PNM Plotter, which doesn't plot in real time,
      only the last frame in the page is meaningful) */
