@@ -6,38 +6,39 @@
 
 int
 #ifdef _HAVE_PROTOS
-_t_closepl (void)
+_t_closepl (S___(Plotter *_plotter))
 #else
-_t_closepl ()
+_t_closepl (S___(_plotter))
+     S___(Plotter *_plotter;)
 #endif
 {
   int retval;
 
   if (!_plotter->open)
     {
-      _plotter->error ("closepl: invalid operation");
+      _plotter->error (R___(_plotter) "closepl: invalid operation");
       return -1;
     }
 
-  _plotter->endpath (); /* flush polyline if any */
+  _plotter->endpath (S___(_plotter)); /* flush polyline if any */
 
   /* pop drawing states in progress, if any, off the stack */
   if (_plotter->drawstate->previous != NULL)
     {
       while (_plotter->drawstate->previous)
-	_plotter->restorestate();
+	_plotter->restorestate (S___(_plotter));
     }
   
-  _tek_move (0, 0);		/* move to lower left corner in Tek space */
-  _tek_mode (MODE_ALPHA);	/* switch to alpha mode, for user's sake */
+  _tek_move (R___(_plotter) 0, 0); /* go to lower left corner in Tek space */
+  _tek_mode (R___(_plotter) MODE_ALPHA); /* switch to alpha mode */
 
   /* remove zeroth drawing state too, so we can start afresh */
 
   /* elements of state that are strings are freed separately */
-  free (_plotter->drawstate->line_mode);
-  free (_plotter->drawstate->join_mode);
-  free (_plotter->drawstate->cap_mode);
-  free (_plotter->drawstate->font_name);
+  free ((char *)_plotter->drawstate->line_mode);
+  free ((char *)_plotter->drawstate->join_mode);
+  free ((char *)_plotter->drawstate->cap_mode);
+  free ((char *)_plotter->drawstate->font_name);
   
   free (_plotter->drawstate);
   _plotter->drawstate = NULL;
@@ -46,14 +47,15 @@ _t_closepl ()
     {
     case D_KERMIT:
       /* use VT340 command to exit graphics mode */
-      _plotter->write_string ("\033[?38l");
+      _plotter->write_string (R___(_plotter) "\033[?38l");
       /* following command may be an alternative */
       /*
 	_plotter->write_string ("\030");
       */
       break;
     case D_XTERM:
-      _plotter->write_string ("\033\003"); /* ESC C-c, restore to VT102 mode */
+      /* ESC C-c, restore to VT102 mode */
+      _plotter->write_string (R___(_plotter) "\033\003"); 
       break;
     case D_GENERIC:
     default:
@@ -61,7 +63,7 @@ _t_closepl ()
     }
   
   /* attempt to flush (will test whether stream is jammed) */
-  retval = _plotter->flushpl ();
+  retval = _plotter->flushpl (S___(_plotter));
 
   _plotter->open = false;	/* flag device as closed */
 
